/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Den Benutzer ein Kartenspiel spielen lassen
*/
function KartenspielTask(
    parentNode, // HTMLElement, in dem der Test dargestellt werden soll
    gegner      // Gegnerart (0 → Mensch, 1 → zufallsziehung durch Computer, 2 → perfekter Computergegner)
) {
    KartenspielTask.parent.call(this);
    this.msgPlaceGeneral = new ScoTools.MessagePlace(true);
    this.msgPlaceGeneral.insertThisBefore(parentNode);
    this.msgPlace = new ScoTools.MessagePlace();
    this.msgPlace.insertThisBefore(parentNode);
    this.node = parentNode;
    this.establish(gegner);
    this.connect();
    AudioFeedbackTask.cache();
    this.render(gegner);
}
KartenspielTask.prototype = Object.create(AudioFeedbackTask.prototype);
KartenspielTask.prototype.constructor = KartenspielTask;
KartenspielTask.parent = AudioFeedbackTask;

// Kartenspiel
KartenspielTask.prototype.spiel = null;

/**
* Kartenspiel anzeigen und starten
* @return undefined
*/
KartenspielTask.prototype.render = function(
    gegner      // Gegnerart (0 → Mensch, 1 → zufallsziehung durch Computer, 2 → perfekter Computergegner)
) {
    var self = this;
    this.node.innerHTML = '';
    this.spiel = new Kartenspiel(this.node, gegner, this.read('studentFirstName'));
    this.spiel.ondone = function(gewonnen, punkte, gegnerPunkte) {
        self.addRestartButton();
        self.reportResult(gewonnen, punkte, gegnerPunkte);
    }
    if (gegner && !this.getDone())
        this.showMsg('Gewinne das Spiel, um die Aufgabe abzuschließen', ScoTools.MSGTYPE_INFO);
}

/**
* Eine Schaltfläche für einen Neustart anzeigen
* @return undefined
*/
KartenspielTask.prototype.addRestartButton = function() {
    var self = this;
    var form = ScoTools.createHtml('form', this.node, { 'method': 'post', 'action': location.href });
    form.addEventListener('submit', function(ev) {
        ev.preventDefault();
        ev.stopPropagation();
        self.spiel.restart();
        form.parentNode.removeChild(form);
    });
    ScoTools.createHtml('button', form, null, 'Neue Spielrunde starten');
}

/**
* Anzahl der bisherigen Versuche auslesen
* @return Number    Anzahl der bisherigen Versuche
*/
KartenspielTask.prototype.getPreviousTries = function() {
    var value = +this.read('score');
    return (Number.isFinite(value) ? value : 0);
}

/**
* Auf Ende des Spiels reagieren
* @return undefined
*/
KartenspielTask.prototype.reportResult = function(
    gewonnen        // Hat der Benutzer gewonnen?
) {
    if (gewonnen)
        this.showMsg('Du hast diese Spielrunde gewonnen.', true);
    else
        this.showMsg('Du hast diese Spielrunde verloren.', false);
    this.playAudio(gewonnen);
    var score = this.getPreviousTries();
    if (gewonnen || !this.spiel.gegner)
        this.saveScore(100 - score, 0, 100, 'passed');
    else
        this.saveScore(Math.min(49, score + 1), 0, 100, 'incomplete');
    this.showFinalMessage();
}

/**
* Richtigkeit der Antworten überprüfen
* @return undefined
*/
KartenspielTask.prototype.evaluate = function() {
    var results = this._questions.map(function(question) { return question.evaluate(); });
    this.needRestart = true;
    var ratio = KartenspielTask.calcRatio(results, this.data.countMode);
    var bestanden = (ratio >= ((this.data.threshold === undefined) ? 1 : this.data.threshold));
    if (this.reportScore)
        this._reportResultScore(bestanden, ratio);
    else
        this._reportResultTries(bestanden, ratio);
    this.playAudio(bestanden);
}
